@@ -7,13 +7,30 @@
 
 from __future__ import absolute_import
 from decimal import Decimal as MyDecimal, ROUND_HALF_EVEN
+from functools import wraps
 
 from marshmallow import validate, utils
 from marshmallow.base import FieldABC, SerializerABC
-from marshmallow.compat import text_type, OrderedDict, iteritems, total_seconds
+from marshmallow.compat import (text_type, OrderedDict, iteritems, total_seconds,
+                                basestring)
 from marshmallow.exceptions import MarshallingError
 
 
+def validated(f):
+    """Decorator that wraps a field's ``format`` or ``output`` method. If an
+    exception is raised during the execution of the wrapped method, a
+    MarshallingError is raised instead with the underlying exception's
+    error message or the user-defined error message (if defined).
+    """
+    @wraps(f)
+    def decorated(self, *args, **kwargs):
+        try:
+            return f(self, *args, **kwargs)
+        except Exception as error:
+            raise MarshallingError(getattr(self, "error", None) or error)
+    return decorated
+
+
 class Marshaller(object):
     '''Callable class responsible for marshalling data and storing errors.
 
@@ -27,40 +44,39 @@ class Marshaller(object):
         self.strict = strict
         self.errors = {}
 
-    def marshal(self, data, fields_dict):
-        """Takes raw data (in the form of a dict, list, object) and a dict of
+    def marshal(self, data, fields_dict, many=False):
+        """Takes raw data (a dict, list, or other object) and a dict of
         fields to output and filters the data based on those fields.
 
         :param data: The actual object(s) from which the fields are taken from
         :param dict fields: A dict whose keys will make up the final serialized
                        response output.
+        :param bool many: Set to ``True`` if ``data`` is a collection object
+                        that is iterable.
+        :returns: An OrderedDict of the marshalled data
         """
-        if utils.is_collection(data):
-            return [self.marshal(d, fields_dict) for d in data]
+        if many and data is not None:
+            return [self.marshal(d, fields_dict, many=False) for d in data]
         items = []
         for attr_name, field_obj in iteritems(fields_dict):
             key = self.prefix + attr_name
             try:
-                if isinstance(field_obj, dict):
-                    item = (key, self.marshal(data, field_obj))
-                else:
-                    try:
-                        item = (key, field_obj.output(attr_name, data))
-                    except TypeError:
-                        # field declared as a class, not an instance
-                        if isinstance(field_obj, type) and \
-                            issubclass(field_obj, FieldABC):
-                            msg = ('Field for "{0}" must be declared as a '
-                                            "Field instance, not a class. "
-                                            'Did you mean "fields.{1}()"?'
-                                            .format(attr_name, field_obj.__name__))
-                            raise TypeError(msg)
-                        raise
+                item = (key, field_obj.output(attr_name, data))
             except MarshallingError as err:  # Store errors
                 if self.strict:
                     raise err
                 self.errors[key] = text_type(err)
                 item = (key, None)
+            except TypeError:
+                # field declared as a class, not an instance
+                if isinstance(field_obj, type) and \
+                    issubclass(field_obj, FieldABC):
+                    msg = ('Field for "{0}" must be declared as a '
+                                    "Field instance, not a class. "
+                                    'Did you mean "fields.{1}()"?'
+                                    .format(attr_name, field_obj.__name__))
+                    raise TypeError(msg)
+                raise
             items.append(item)
         return OrderedDict(items)
 
@@ -107,11 +123,13 @@ class Raw(FieldABC):
     :param default: Default value for the field if the attribute is not set.
     :param str attribute: The name of the attribute to get the value from. If
         ``None``, assumes the attribute has the same name as the field.
+    :param str error: Error message stored upon validation failure.
     """
 
-    def __init__(self, default=None, attribute=None):
+    def __init__(self, default=None, attribute=None, error=None):
         self.attribute = attribute
         self.default = default
+        self.error = error
 
     def get_value(self, key, obj):
         '''Return the value for a given key from an object.'''
@@ -145,32 +163,49 @@ class Raw(FieldABC):
 
 class Nested(Raw):
     """Allows you to nest a :class:`Serializer <marshmallow.Serializer>`
-    or set of fields inside a field.
+    inside a field.
 
-    Example: ::
+    Examples: ::
 
         user = fields.Nested(UserSerializer)
+        collaborators = fields.Nested(UserSerializer, many=True, only='id')
+        parent = fields.Nested('self')
 
-    :param Serializer nested: The Serializer class or instance to nest.
-    :param iterable exclude: A list or tuple of fields to exclude.
-    :param iterable only: A list or tuple of fields to marshal. If ``None``, all fields
-        will be marshalled. Takes precedence over ``exclude``.
+    :param Serializer nested: The Serializer class or instance to nest, or
+        "self" to nest the serializer within itself.
+    :param tuple exclude: A list or tuple of fields to exclude.
+    :param only: A tuple or string of the field(s) to marshal. If ``None``, all fields
+        will be marshalled. If a field name (string) is given, only a single
+        value will be returned as output instead of a dictionary.
+        This parameter takes precedence over ``exclude``.
     :param bool allow_null: Whether to return None instead of a dictionary
         with null keys, if a nested dictionary has all-null keys
+    :param bool many: Whether the field is a collection of objects.
     """
 
-    def __init__(self, nested, exclude=None, only=None, allow_null=False, **kwargs):
+    def __init__(self, nested, exclude=None, only=None, allow_null=False,
+                many=False, **kwargs):
         self.nested = nested
         self.allow_null = allow_null
         self.only = only
-        self.exclude = exclude
-        self.serializer = None
+        self.exclude = exclude or ()
+        self.many = many
+        self.__serializer = None
+        self.__updated_fields = False  # ensures serializer fields are updated
+                                        # only once
         super(Nested, self).__init__(**kwargs)
 
     def __get_fields_to_marshal(self, all_fields):
-        '''Filter the all_fields based on self.only and self.exclude.'''
+        '''Filter all_fields based on self.only and self.exclude.'''
         # Default 'only' to all the nested fields
-        only = set(all_fields) if self.only is None else set(self.only)
+        ret = OrderedDict()
+        if all_fields is None:
+            return ret
+        elif isinstance(self.only, basestring):
+            ret[self.only] = all_fields[self.only]
+            return ret
+        else:
+            only = set(all_fields) if self.only is None else set(self.only)
         if self.exclude and self.only:
             # Make sure that only takes precedence
             exclude = set(self.exclude) - only
@@ -180,27 +215,57 @@ class Nested(Raw):
                     if k in only and k not in exclude)
         return OrderedDict(filtered)
 
+    @property
+    def serializer(self):
+        """The nested Serializer object."""
+        # Cache the serializer instance
+        if not self.__serializer:
+            if isinstance(self.nested, SerializerABC):
+                self.__serializer = self.nested
+            elif isinstance(self.nested, type) and \
+                    issubclass(self.nested, SerializerABC):
+                self.__serializer = self.nested(None, many=self.many)
+            elif self.nested == 'self':
+                self.__serializer = self.parent  # The serializer this fields belongs to
+                # For now, don't allow nesting of depth > 1
+                self.exclude += (self.name, )  # Exclude this field
+            else:
+                raise ValueError("Nested fields must be passed a Serializer, not {0}."
+                                .format(self.nested.__class__))
+        return self.__serializer
+
     def output(self, key, obj):
         nested_obj = self.get_value(key, obj)
         if self.allow_null and nested_obj is None:
             return None
-        if isinstance(self.nested, SerializerABC):
-            self.serializer = self.nested
-            self.nested._data = nested_obj
-        elif isinstance(self.nested, type) and \
-                issubclass(self.nested, SerializerABC):
-            self.serializer = self.nested(nested_obj)
-        else:
-            raise ValueError("Nested fields must be passed a Serializer, not {0}."
-                            .format(self.nested.__class__))
+        self.serializer.many = self.many
+        self.serializer.obj = nested_obj
+        if not self.__updated_fields:
+            self.__updated_fields = True
+            self.serializer._update_fields(nested_obj)
         fields = self.__get_fields_to_marshal(self.serializer.fields)
-        ret = self.serializer.marshal(nested_obj, fields)
+        ret = self.serializer.marshal(nested_obj, fields, many=self.many)
         # Parent should get any errors stored after marshalling
         if self.serializer.errors:
             self.parent.errors[key] = self.serializer.errors
+        if isinstance(self.only, basestring):  # self.only is a field name
+            if self.many:
+                return flatten(ret, key=self.only)
+            else:
+                return ret[self.only]
         return ret
 
 
+def flatten(dictlist, key):
+    """Flattens a list of dicts into just a list of values.
+    ::
+
+        >>> d = [{'id': 1, 'name': 'foo'}, {'id': 2, 'name': 'bar'}]
+        >>> flatten(d, 'id')
+        [1, 2]
+    """
+    return [d[key] for d in dictlist]
+
 class List(Raw):
     '''A list field.
 
@@ -242,14 +307,14 @@ class List(Raw):
 class String(Raw):
     """A string field."""
 
-    def __init__(self, default='', attribute=None):
-        return super(String, self).__init__(default, attribute)
+    def __init__(self, default='', attribute=None, *args, **kwargs):
+        return super(String, self).__init__(default, attribute, *args, **kwargs)
 
     def format(self, value):
         try:
             return text_type(value)
         except ValueError as ve:
-            raise MarshallingError(ve)
+            raise MarshallingError(self.error or ve)
 
 
 class UUID(String):
@@ -257,14 +322,15 @@ class UUID(String):
     pass
 
 
-class NumberField(Raw):
+class Number(Raw):
     '''Base class for number fields.'''
 
-    num_type = int
+    num_type = float
 
-    def __init__(self, default=0, attribute=None, as_string=False):
+    def __init__(self, default=0.0, attribute=None, as_string=False, error=None, **kwargs):
         self.as_string = as_string
-        super(NumberField, self).__init__(default=default, attribute=attribute)
+        super(Number, self).__init__(default=default, attribute=attribute,
+            error=error, **kwargs)
 
     def _format_num(self, value):
         '''Return the correct value for a number, given the passed in
@@ -284,7 +350,7 @@ class NumberField(Raw):
             raise MarshallingError(ve)
 
 
-class Integer(NumberField):
+class Integer(Number):
     """An integer field.
 
     :param bool as_string: If True, format the value as a string.
@@ -312,7 +378,7 @@ class FormattedString(Raw):
             raise MarshallingError(error)
 
 
-class Float(NumberField):
+class Float(Number):
     """
     A double as IEEE-754 double precision string.
 
@@ -322,7 +388,7 @@ class Float(NumberField):
     num_type = float
 
 
-class Arbitrary(NumberField):
+class Arbitrary(Number):
     """A floating point number with an arbitrary precision,
     formatted as as string.
     ex: 634271127864378216478362784632784678324.23432
@@ -361,16 +427,14 @@ class DateTime(Raw):
         super(DateTime, self).__init__(default=default, attribute=attribute)
         self.dateformat = format
 
+    @validated
     def format(self, value):
         self.dateformat = self.dateformat or 'rfc'
-        try:
-            format_func = DATEFORMAT_FUNCTIONS.get(self.dateformat, None)
-            if format_func:
-                return format_func(value, localtime=self.localtime)
-            else:
-                return value.strftime(self.dateformat)
-        except AttributeError as ae:
-            raise MarshallingError(ae)
+        format_func = DATEFORMAT_FUNCTIONS.get(self.dateformat, None)
+        if format_func:
+            return format_func(value, localtime=self.localtime)
+        else:
+            return value.strftime(self.dateformat)
 
 
 class LocalDateTime(DateTime):
@@ -429,19 +493,19 @@ class TimeDelta(Raw):
 ZERO = MyDecimal()
 
 
-class Fixed(NumberField):
+class Fixed(Number):
     """A fixed-precision number as a string.
     """
 
-    def __init__(self, decimals=5, default=0, attribute=None):
-        super(Fixed, self).__init__(default=default, attribute=attribute)
+    def __init__(self, decimals=5, default=0, attribute=None, error=None,
+                 *args, **kwargs):
+        super(Fixed, self).__init__(default=default, attribute=attribute, error=error,
+                            *args, **kwargs)
         self.precision = MyDecimal('0.' + '0' * (decimals - 1) + '1')
 
+    @validated
     def format(self, value):
-        try:
-            dvalue = utils.float_to_decimal(float(value))
-        except ValueError as ve:
-            raise MarshallingError(ve)
+        dvalue = utils.float_to_decimal(float(value))
         if not dvalue.is_normal() and dvalue != ZERO:
             raise MarshallingError('Invalid Fixed precision number.')
         return text_type(dvalue.quantize(self.precision, rounding=ROUND_HALF_EVEN))
@@ -460,31 +524,29 @@ class Url(Raw):
         ``None``, assumes the attribute has the same name as the field.
     :param bool relative: Allow relative URLs.
     """
-    def __init__(self, default=None, attribute=None, relative=False):
-        super(Url, self).__init__(default=default, attribute=attribute)
+    def __init__(self, default=None, attribute=None, relative=False, *args, **kwargs):
+        super(Url, self).__init__(default=default, attribute=attribute,
+                *args, **kwargs)
         self.relative = relative
 
+    @validated
     def output(self, key, obj):
         value = self.get_value(key, obj)
         if value is None:
             return self.default
-        try:
-            return validate.url(value, relative=self.relative)
-        except Exception as err:
-            raise MarshallingError(err)
+        return validate.url(value, relative=self.relative)
 
 
 class Email(Raw):
     """A validated email field.
     """
+
+    @validated
     def output(self, key, obj):
         value = self.get_value(key, obj)
         if value is None:
             return self.default
-        try:
-            return validate.email(value)
-        except Exception as err:
-            raise MarshallingError(err)
+        return validate.email(value)
 
 
 class Method(Raw):
@@ -524,3 +586,22 @@ class Function(Raw):
             raise MarshallingError(te)
         except AttributeError:
             pass
+
+
+class Select(Raw):
+    """A field that only lets you set values in the selection.
+
+    :param choices: A list of valid values.
+    :param default: Default value for the field if the attribute is not set.
+    :param str attribute: The name of the attribute to get the value from. If
+        ``None``, assumes the attribute has the same name as the field.
+    :param str error: Error message stored upon validation failure.
+    """
+    def __init__(self,  choices, default=None, attribute=None, error=None):
+        self.choices = choices
+        return super(Select, self).__init__(default, attribute, error)
+
+    def format(self, value):
+        if value not in self.choices:
+            raise MarshallingError("'%s' is not a valid choice for this field." % value)
+        return value
