@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 '''The Serializer class, including its metaclass and options (class Meta).'''
-from __future__ import absolute_import, print_function
+from __future__ import absolute_import
 import datetime as dt
 import json
 import copy
 import uuid
 import types
+import warnings
 
 from marshmallow import base, fields, utils
 from marshmallow.compat import (with_metaclass, iteritems, text_type,
@@ -15,7 +16,7 @@ from marshmallow.compat import (with_metaclass, iteritems, text_type,
 class SerializerMeta(type):
     '''Metaclass for the Serializer class. Binds the declared fields to
     a ``_declared_fields`` attribute, which is a dictionary mapping attribute
-    names to field classes and instances.
+    names to field objects.
     '''
 
     def __new__(mcs, name, bases, attrs):
@@ -47,17 +48,63 @@ class SerializerOpts(object):
 
     def __init__(self, meta):
         self.fields = getattr(meta, 'fields', ())
+        if not isinstance(self.fields, (list, tuple)):
+            raise ValueError("`fields` option must be a list or tuple.")
         self.additional = getattr(meta, 'additional', ())
+        if not isinstance(self.additional, (list, tuple)):
+            raise ValueError("`additional` option must be a list or tuple.")
         if self.fields and self.additional:
             raise ValueError("Cannot set both `fields` and `additional` options"
                             " for the same serializer.")
         self.exclude = getattr(meta, 'exclude', ())
+        if not isinstance(self.exclude, (list, tuple)):
+            raise ValueError("`exclude` must be a list or tuple.")
         self.strict = getattr(meta, 'strict', False)
         self.dateformat = getattr(meta, 'dateformat', None)
 
 
 class BaseSerializer(base.SerializerABC):
-    '''Base serializer class which defines the interface for a serializer.
+    '''Base serializer class with which to define custom serializers.
+
+    Example usage:
+
+    .. code-block:: python
+
+        from datetime import datetime
+        from marshmallow import Serializer, fields
+
+        class Person(object):
+            def __init__(self, name):
+                self.name = name
+                self.date_born = datetime.now()
+
+        class PersonSerializer(Serializer):
+            name = fields.String()
+            date_born = fields.DateTime()
+
+        # Or, equivalently
+        class PersonSerializer2(Serializer):
+            class Meta:
+                fields = ("name", "date_born")
+
+        person = Person("Guido van Rossum")
+        serialized = PersonSerializer(person)
+        serialized.data
+        # OrderedDict([('name', u'Guido van Rossum'),
+        #                ('date_born', 'Sat, 09 Nov 2013 00:10:29 -0000')])
+
+    :param obj: The object or collection of objects to be serialized.
+    :param dict extra: A dict of extra attributes to bind to the serialized result.
+    :param tuple only: A list or tuple of fields to serialize. If ``None``, all
+        fields will be serialized.
+    :param tuple exclude: A list or tuple of fields to exclude from the
+        serialized result.
+    :param str prefix: Optional prefix that will be prepended to all the
+        serialized field names.
+    :param bool strict: If ``True``, raise errors if invalid data are passed in
+        instead of failing silently and storing the errors.
+    :param bool many: Should be set to ``True`` if ``obj`` is a collection
+        so that the object will be serialized to a list.
     '''
     TYPE_MAPPING = {
         text_type: fields.String,
@@ -99,134 +146,123 @@ class BaseSerializer(base.SerializerABC):
         pass
 
     def __init__(self, obj=None, extra=None, only=None,
-                exclude=None, prefix='', strict=False):
+                exclude=None, prefix='', strict=False, many=False):
+        if not many and utils.is_collection(obj):
+            warnings.warn('Implicit collection handling is deprecated. Set '
+                            'many=True to serialize a collection.',
+                            category=DeprecationWarning)
+        # copy declared fields from metaclass
+        self.declared_fields = copy.deepcopy(self._declared_fields)
+        self.fields = OrderedDict()
+        self.__data = None
+        self.obj = obj
+        self.many = many
         self.opts = SerializerOpts(self.Meta)
+        self.only = only or ()
+        self.exclude = exclude or ()
+        self.prefix = prefix
+        self.strict = strict or self.opts.strict
+        #: Callable marshalling object
+        self.marshal = fields.Marshaller(prefix=self.prefix, strict=self.strict)
+        self.extra = extra
         if isinstance(obj, types.GeneratorType):
             self.obj = list(obj)
         else:
             self.obj = obj
-        self.only = only or ()
-        self.exclude = exclude or ()
-        self.prefix = prefix
-        self.fields = self.__get_fields()  # Dict of fields
-        if strict or self.opts.strict:
-            self.strict = True
-        else:
-            self.strict = False
-        #: Callable marshalling object
-        self._marshal = fields.Marshaller(prefix=self.prefix, strict=self.strict)
-        # Store the marshalled data
-        # MUST occur upon initialization
-        self._data = self.marshal(self.obj, self.fields)
-        if extra:
-            self._data.update(extra)
+        self._update_fields(obj)
+        # If object is passed in, marshal it immediately so that errors are stored
+        if self.obj is not None:
+            self.__data = self.marshal(self.obj, self.fields, many=self.many)
+            if self.extra:
+                self.__data.update(self.extra)
 
-    def __get_fields(self):
-        '''Return the declared fields for the object as an OrderedDict.'''
-        ret = OrderedDict()
-        declared_fields = copy.deepcopy(self._declared_fields)  # Copy _declared_fields
-                                                                # from metaclass
-        # Explicitly declared fields
-        for field_name, field_obj in iteritems(declared_fields):
-            ret[field_name] = field_obj
+    def _update_fields(self, obj):
+        '''Update fields based on the passed in object.'''
+        # if only __init__ param is specified, only return those fields
+        if self.only:
+            ret = self.__filter_fields(self.only)
+            self.__set_field_attrs(ret)
+            self.fields = ret
+            return self.fields
 
         if self.opts.fields:
             # Return only fields specified in fields option
-            ret = self.__get_opts_fields(ret, self.opts.fields)
+            field_names = set(self.opts.fields)
         elif self.opts.additional:
             # Return declared fields + additional fields
-            field_names = tuple(ret.keys()) + tuple(self.opts.additional)
-            ret = self.__get_opts_fields(ret, field_names)
-
-        # if only __init__ param is specified, only return those fields
-        if self.only:
-            filtered = OrderedDict()
-            for field_name in self.only:
-                if field_name not in ret:
-                    raise AttributeError(
-                        '"{0}" is not a valid field for {1}.'
-                            .format(field_name, self.obj))
-                filtered[field_name] = ret[field_name]
-            self.__initialize_fields(filtered)
-            return filtered
+            field_names = set(self.declared_fields.keys()) | set(self.opts.additional)
+        else:
+            field_names = set(self.declared_fields.keys())
 
         # If "exclude" option or param is specified, remove those fields
-        if not isinstance(self.opts.exclude, (list, tuple)) or \
-                            not isinstance(self.exclude, (list, tuple)):
-            raise ValueError("`exclude` must be a list or tuple.")
-        excludes = set(self.opts.exclude + self.exclude)
+        excludes = set(self.opts.exclude) | set(self.exclude)
         if excludes:
-            for field_name in excludes:
-                ret.pop(field_name, None)
+            field_names = field_names - excludes
+        ret = self.__filter_fields(field_names)
         # Set parents
-        self.__initialize_fields(ret)
-        return ret
+        self.__set_field_attrs(ret)
+        self.fields = ret
+        return self.fields
 
-    def __initialize_fields(self, fields_dict):
+    def __set_field_attrs(self, fields_dict):
         '''Set the parents of all field objects in fields_dict to self, and
         set the dateformat specified in ``class Meta``, if necessary.
         '''
-        for _, field_obj in iteritems(fields_dict):
+        for field_name, field_obj in iteritems(fields_dict):
             if not field_obj.parent:
                 field_obj.parent = self
+            if not field_obj.name:
+                field_obj.name = field_name
             if isinstance(field_obj, fields.DateTime):
                 if field_obj.dateformat is None:
                     field_obj.dateformat = self.opts.dateformat
         return fields_dict
 
-    def __get_opts_fields(self, declared_fields, field_names):
+    def __filter_fields(self, field_names):
         '''Return only those field_name:field_obj pairs specified by
         ``field_names``.
 
-        :param dict declared_fields: The original dictionary of explicitly
-            declared fields.
-        :param tuple field_names: List of field names to include in the final
+        :returns: An OrderedDict of field_name:field_obj pairs.
+
+        :param set field_names: Field names to include in the final
             return dictionary.
         '''
         # Convert obj to a dict
-        if not isinstance(self.opts.fields, (list, tuple)):
-            raise ValueError("`fields` option must be a list or tuple.")
         obj_marshallable = utils.to_marshallable_type(self.obj)
-        if utils.is_collection(obj_marshallable):  # Homogeneous collection
-            try:
+        if obj_marshallable and self.many:
+            try:  # Homogeneous collection
                 obj_dict = utils.to_marshallable_type(obj_marshallable[0])
             except IndexError:  # Nothing to serialize
-                return declared_fields
+                return self.declared_fields
         else:
             obj_dict = obj_marshallable
         ret = OrderedDict()
         for key in field_names:
-            if key in declared_fields:
-                ret[key] = declared_fields[key]
+            if key in self.declared_fields:
+                ret[key] = self.declared_fields[key]
             else:
-                try:
-                    if obj_dict:
+                if obj_dict:
+                    try:
                         attribute_type = type(obj_dict[key])
-                        field_obj = self.TYPE_MAPPING.get(attribute_type, fields.Raw)()
-                    else:  # Object is None
-                        field_obj = fields.Raw()
-                except KeyError:
-                    raise AttributeError(
-                        '"{0}" is not a valid field for {1}.'.format(key, self.obj))
+                    except KeyError:
+                        raise AttributeError(
+                            '"{0}" is not a valid field for {1}.'.format(key, self.obj))
+                    field_obj = self.TYPE_MAPPING.get(attribute_type, fields.Raw)()
+                else:  # Object is None
+                    field_obj = fields.Raw()
                 # map key -> field (default to Raw)
                 ret[key] = field_obj
         return ret
 
-    def marshal(self, data, fields_dict):
-        """Takes the data (a dict, list, or object) and a dict of fields.
-        Stores any errors that occur.
-
-        :param data: The actual object(s) from which the fields are taken from
-        :param dict fields_dict: A dict whose keys will make up the final serialized
-                       response output
-        """
-        return self._marshal(data, fields_dict)
-
     @property
     def data(self):
         '''The serialized data as an ``OrderedDict``.
         '''
-        return self._data
+        if not self.__data:  # Cache the data
+            self.__data = self.marshal(self.obj, self.fields, many=self.many)
+            if self.extra:
+                self.__data.update(self.extra)
+        return self.__data
 
     @property
     def json(self):
@@ -236,7 +272,7 @@ class BaseSerializer(base.SerializerABC):
     @property
     def errors(self):
         '''Dictionary of errors raised during serialization.'''
-        return self._marshal.errors
+        return self.marshal.errors
 
     def to_json(self, *args, **kwargs):
         '''Return the JSON representation of the data. Takes the same arguments
@@ -268,44 +304,4 @@ class BaseSerializer(base.SerializerABC):
 
 
 class Serializer(with_metaclass(SerializerMeta, BaseSerializer)):
-    '''Base serializer class with which to define custom serializers.
-
-    Example usage:
-
-    .. code-block:: python
-
-        from datetime import datetime
-        from marshmallow import Serializer, fields
-
-        class Person(object):
-            def __init__(self, name):
-                self.name = name
-                self.date_born = datetime.now()
-
-        class PersonSerializer(Serializer):
-            name = fields.String()
-            date_born = fields.DateTime()
-
-        # Or, equivalently
-        class PersonSerializer2(Serializer):
-            class Meta:
-                fields = ("name", "date_born")
-
-        person = Person("Guido van Rossum")
-        serialized = PersonSerializer(person)
-        serialized.data
-        # OrderedDict([('name', u'Guido van Rossum'),
-        #                ('date_born', 'Sat, 09 Nov 2013 00:10:29 -0000')])
-
-    :param obj: The object or collection of objects to be serialized.
-    :param dict extra: A dict of extra attributes to bind to the serialized result.
-    :param tuple only: A list or tuple of fields to serialize. If ``None``, all
-        fields will be serialized.
-    :param tuple exclude: A list or tuple of fields to exclude from the
-        serialized result.
-    :param str prefix: Optional prefix that will be prepended to all the
-        serialized field names.
-    :param bool strict: If ``True``, raise errors if invalid data are passed in
-        instead of failing silently and storing the errors.
-    '''
-    pass
+    __doc__ = BaseSerializer.__doc__
