@@ -5,6 +5,7 @@ import json
 import datetime
 from email.utils import formatdate
 from calendar import timegm
+import types
 
 from decimal import Decimal, Context, Inexact
 from pprint import pprint as py_pprint
@@ -36,6 +37,7 @@ def is_instance_or_subclass(val, class_):
     except TypeError:
         return isinstance(val, class_)
 
+
 def float_to_decimal(f):
     """Convert a floating point number to a Decimal with no loss of information.
         See: http://docs.python.org/release/2.6.7/library/decimal.html#decimal-faq
@@ -57,11 +59,15 @@ def to_marshallable_type(obj):
     if obj is None:
         return None  # make it idempotent for None
 
+    if hasattr(obj, '__marshallable__'):
+        return obj.__marshallable__()
+
     if hasattr(obj, '__getitem__'):
         return obj  # it is indexable it is ok
 
-    if hasattr(obj, '__marshallable__'):
-        return obj.__marshallable__()
+    if isinstance(obj, types.GeneratorType):
+        return list(obj)
+
     return dict([(name, getattr(obj, name, None))  for name in dir(obj)
                 if not name.startswith("__") and not name.endswith("__")])
 
